#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <psapi.h>
#include <global.h>
#include <md5.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <io.h>
#include <util.h>

//
// Calculates the md5 of a file.
//
bool get_md5_of_file( const char* filename, unsigned char md5[16] )
{
    FILE* fp = fopen( filename, "rb");
    if( !fp )
	return false;

    fseek( fp, 0, SEEK_END );
    unsigned size = ftell(fp);
    rewind(fp);

    unsigned char* buf = new unsigned char[size];
    if( fread( buf, size, 1, fp ) != 1 )
    {
	fclose(fp);
	delete[] buf;
	return false;
    }
    fclose(fp);

    MD5_CTX md5_ctx;
    MD5Init( &md5_ctx );
    MD5Update( &md5_ctx, buf, size );
    MD5Final( md5, &md5_ctx );
    delete[] buf;
    return true;
}

//
// Calculates the md5 of a process executable.
// 
bool get_md5_of_process( DWORD pid, unsigned char md5[16] )
{
    HANDLE process = OpenProcess( PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, false, pid );
    if( !process )
	return false;

    char buf[4096];
    if( !GetModuleFileNameEx( process, NULL, buf, sizeof(buf) ) )
    {
	CloseHandle( process );
	return false;
    }
    CloseHandle( process );

    return get_md5_of_file( buf, md5 );
}

void log(const char* format, ... )
{
#ifdef DEBUG    
    char buf[4096];
    va_list va;
    va_start( va, format );
    _vsnprintf_s(buf, sizeof(buf), sizeof(buf)-1, format, va );
    va_end( va );
    printf("%s", buf);
#endif    
}

std::string strprintf(const char* format, ... )
{
    char buf[4096];
    va_list va;
    va_start( va, format );
    _vsnprintf_s(buf, sizeof(buf), sizeof(buf)-1, format, va );
    va_end( va );
    return buf;
}

//
// Initializes a console used for debug output
//
void init_console()
{
    CONSOLE_SCREEN_BUFFER_INFO coninfo;
    const DWORD max_lines = 1000;
    AllocConsole();
    GetConsoleScreenBufferInfo( GetStdHandle( STD_OUTPUT_HANDLE ), &coninfo );
    coninfo.dwSize.Y = max_lines;
    SetConsoleScreenBufferSize( GetStdHandle( STD_OUTPUT_HANDLE ), coninfo.dwSize );

    long std_handle = (long)GetStdHandle(STD_OUTPUT_HANDLE);
    int con_handle = _open_osfhandle(std_handle, _O_TEXT);
    FILE* fp = _fdopen( con_handle, "w");
    *stdout = *fp;
    setvbuf( stdout, NULL, _IONBF, 0);

    std_handle = (long)GetStdHandle(STD_INPUT_HANDLE);
    con_handle = _open_osfhandle(std_handle, _O_TEXT);
    fp = _fdopen( con_handle, "r");
    *stdin = *fp;
    setvbuf( stdin, NULL, _IONBF, 0);

    std_handle = (long)GetStdHandle(STD_ERROR_HANDLE);
    con_handle = _open_osfhandle(std_handle, _O_TEXT);
    fp = _fdopen( con_handle, "w");
    *stderr = *fp;
    setvbuf( stdout, NULL, _IONBF, 0);
}

